package com.cicdi.notify.template;

import com.cicdi.notify.NotifyType;

import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

/**
 * 模板管理器
 *
 * @author xueye
 */
public abstract class AbstractTemplateManager implements TemplateManager {

    protected Map<String, Map<String, TemplateProvider>> providers = new ConcurrentHashMap<>();

    protected Map<String, Template> templates = new ConcurrentHashMap<>();

    protected abstract TemplateProperties getProperties(NotifyType type, String id);

    /**
     * 注册模板提供商
     *
     * @param provider 模板提供商
     */
    @Override
    public void register(TemplateProvider provider) {
        providers.computeIfAbsent(provider.getType().getId(), ignore -> new ConcurrentHashMap<>()).put(provider.getProvider().getId(), provider);
    }

    @Override
    public Template createTemplate(NotifyType type, TemplateProperties prop) {
        Map<String, TemplateProvider> providerMap = providers.get(type.getId());
        if (null == providerMap) {
            throw new UnsupportedOperationException("不支持的通知类型: " + prop.getType());
        }
        TemplateProvider templateProvider = providerMap.get(prop.getProvider());
        if (null == templateProvider) {
            throw new UnsupportedOperationException("不支持的提供商: " + prop.getProvider());
        }
        return templateProvider.createTemplate(prop);
    }


    @Override
    public Template getTemplate(NotifyType type, String id) {
        Template template = templates.get(id);
        if (null == template) {
            TemplateProperties properties = this.getProperties(type, id);
            template = this.createTemplate(type, properties);
            if (null == template) {
                throw new UnsupportedOperationException("通知类型不支持: " + type.getId());
            }
            templates.put(id, template);
        }
        return template;
    }

    @Override
    public void reload(String templateId) {
        templates.remove(templateId);
    }
}
